(herald "Four Party Needham-Schroeder-Lowe Protocol")

(comment "CPSA 2.2.3")
(comment "All input read")

(defprotocol nsl4cm basic
  (defrole init
    (vars (a b c d name) (na nb nc nd text))
    (trace (send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b)))))
  (defrole resp1
    (vars (a b c d name) (na nb nc nd text))
    (trace (recv (enc a c d na (pubk b)))
      (send (enc a b d na nb (pubk c))) (recv (enc nb nc nd (pubk b)))
      (send (enc nc nd (pubk c)))))
  (defrole resp2
    (vars (a b c d name) (na nb nc nd text))
    (trace (recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))) (recv (enc nc nd (pubk c)))
      (send (enc nd (pubk d)))))
  (defrole resp3
    (vars (a b c d name) (na nb nc nd text))
    (trace (recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a)))
      (recv (enc nd (pubk d))))))

(defskeleton nsl4cm
  (vars (na nb nc nd text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b)))))
  (label 0)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1)
  (parent 0)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 2)
  (parent 1)
  (unrealized (0 1) (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (2 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 3)
  (parent 2)
  (seen 4)
  (unrealized (0 1) (2 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 3 1 resp1 2) na (2 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 4)
  (parent 2)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 3 1 resp1 2) na (2 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 5)
  (parent 2)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 4 1 resp1 2) na (2 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 6)
  (parent 3)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (2 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 7)
  (parent 3)
  (unrealized (0 1) (2 0) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 8)
  (parent 4)
  (unrealized (0 1) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 9)
  (parent 5)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 10)
  (parent 5)
  (seen 20)
  (unrealized (0 1) (3 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 11)
  (parent 6)
  (seen 27)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 12)
  (parent 6)
  (seen 32)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 13)
  (parent 7)
  (seen 15)
  (unrealized (0 1) (2 0) (4 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 14)
  (parent 7)
  (unrealized (0 1) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 3 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 15)
  (parent 7)
  (seen 39)
  (unrealized (0 1) (2 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 16)
  (parent 7)
  (seen 36)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 5 3 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 17)
  (parent 7)
  (seen 46)
  (unrealized (0 1) (2 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 18)
  (parent 8)
  (unrealized (0 1) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 4 1 resp1 2) na (3 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 19)
  (parent 8)
  (seen 21 56)
  (unrealized (0 1) (3 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 4 1 resp1 2) na (3 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 20)
  (parent 8)
  (seen 25 54)
  (unrealized (0 1) (3 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 4 2 resp2 2) na (3 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 21)
  (parent 8)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 22)
  (parent 9)
  (seen 64)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 23)
  (parent 9)
  (seen 24 68)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (3 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 24)
  (parent 10)
  (seen 54 60)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 4 2 resp2 2) nb-0 (3 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 25)
  (parent 10)
  (seen 70)
  (unrealized (0 1))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 26)
  (parent 11)
  (seen 28 41 76)
  (unrealized (0 1) (4 0))
  (comment "8 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 27)
  (parent 11)
  (seen 5 30 41 76)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 28)
  (parent 11)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 29)
  (parent 11)
  (seen 9 10)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 30)
  (parent 11)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 31)
  (parent 12)
  (seen 88)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 32)
  (parent 12)
  (seen 35 88 91)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 33)
  (parent 12)
  (seen 35 60 93)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 34)
  (parent 12)
  (seen 54 59 60)
  (unrealized (0 1) (4 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 35)
  (parent 12)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (4 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 36)
  (parent 13)
  (seen 47)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (4 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 37)
  (parent 13)
  (seen 41 100)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 38)
  (parent 13)
  (unrealized (0 1) (2 0) (4 0) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 39)
  (parent 14)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 3 resp1 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 40)
  (parent 14)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 5 3 resp1 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 41)
  (parent 14)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 42)
  (parent 14)
  (unrealized (0 1) (4 0) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 1 resp1 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 43)
  (parent 15)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 44)
  (parent 15)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 45)
  (parent 15)
  (unrealized (0 1) (2 0) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb-0)) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 46)
  (parent 16)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 47)
  (parent 16)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-1 (4 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 48)
  (parent 16)
  (seen 136 137 138)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 49)
  (parent 16)
  (seen 124)
  (unrealized (0 1) (4 0) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 50)
  (parent 17)
  (seen 51)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 5 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 51)
  (parent 17)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 52)
  (parent 17)
  (seen 136 137 138)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 53)
  (parent 17)
  (seen 134)
  (unrealized (0 1) (2 0) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) na (3 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 54)
  (parent 18)
  (seen 60 72)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 55)
  (parent 18)
  (unrealized (0 1) (3 0) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 2 resp2 2) na (3 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 56)
  (parent 18)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) na (3 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 57)
  (parent 18)
  (seen 61 62)
  (unrealized (0 1))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 58)
  (parent 19)
  (unrealized (0 1) (3 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (3 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 59)
  (parent 20)
  (seen 167)
  (unrealized (0 1) (3 0) (4 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 4 2 resp2 2) nb-0 (3 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 60)
  (parent 20)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a)))))
  (label 61)
  (parent 21)
  (seen 163)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 62)
  (parent 21)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 63)
  (parent 22)
  (seen 17 65 178)
  (unrealized (0 1) (4 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 64)
  (parent 22)
  (seen 5 17 29 67 178)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 65)
  (parent 22)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 66)
  (parent 22)
  (seen 187)
  (unrealized (0 1))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 67)
  (parent 22)
  (seen 181)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 68)
  (parent 23)
  (seen 54 70 172)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 69)
  (parent 23)
  (seen 60)
  (unrealized (0 1) (4 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 70)
  (parent 23)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (3 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 71)
  (parent 24)
  (seen 160)
  (unrealized (0 1) (3 0) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-1 (3 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 72)
  (parent 24)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nc nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb nc) (nc-0 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b d nc nc (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a)))))
  (label 73)
  (parent 25)
  (seen 171)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 74)
  (parent 25)
  (seen 175)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (4 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 75)
  (parent 26)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 76)
  (parent 26)
  (seen 6 41 79 80)
  (unrealized (0 1) (4 0))
  (comment "9 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-2 (4 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 77)
  (parent 26)
  (seen 11 12)
  (unrealized (0 1))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 78)
  (parent 26)
  (seen 218)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-2 (4 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 79)
  (parent 26)
  (seen 85 86 87)
  (unrealized (0 1))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 80)
  (parent 27)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-1 (4 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 81)
  (parent 27)
  (seen 9 10)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 82)
  (parent 27)
  (seen 228)
  (unrealized (0 1) (4 0) (5 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 83)
  (parent 28)
  (seen 233)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 84)
  (parent 28)
  (seen 239)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 85)
  (parent 30)
  (seen 245)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 86)
  (parent 30)
  (seen 251)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 87)
  (parent 30)
  (seen 257)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 88)
  (parent 31)
  (seen 91 92)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-2 (4 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 89)
  (parent 31)
  (seen 60 154 155 156)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 90)
  (parent 31)
  (seen 266)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 91)
  (parent 31)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-2 (4 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 92)
  (parent 31)
  (seen 97 98 99)
  (unrealized (0 1))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 93)
  (parent 32)
  (seen 35 60 91)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 94)
  (parent 32)
  (seen 278)
  (unrealized (0 1) (4 0) (5 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 95)
  (parent 33)
  (seen 283)
  (unrealized (0 1) (4 0) (5 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 96)
  (parent 34)
  (seen 70 73 74)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nc nb nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-0 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b d nc nc (pubk c))
    (enc a b d nc nb (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a)))))
  (label 97)
  (parent 35)
  (seen 73)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 98)
  (parent 35)
  (seen 288)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 99)
  (parent 35)
  (seen 236 254 290 294)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 100)
  (parent 36)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-2 (4 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 101)
  (parent 36)
  (seen 296 297 298)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 102)
  (parent 36)
  (seen 141 142 299)
  (unrealized (0 1) (4 0) (6 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 103)
  (parent 36)
  (seen 112 301)
  (unrealized (0 1) (4 0) (6 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 104)
  (parent 37)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-2 (2 0)
    (enc a b c nb-2 nb-2 nc-1 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 105)
  (parent 37)
  (seen 296 297 298)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 106)
  (parent 37)
  (seen 311)
  (unrealized (0 1) (2 0) (6 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 107)
  (parent 37)
  (seen 318)
  (unrealized (0 1) (2 0) (6 0))
  (comment "8 in cohort - 7 not yet seen"))

(comment "Strand bound exceeded--aborting run")

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 108)
  (parent 38)
  (unrealized (0 1) (2 0) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 109)
  (parent 38)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 110)
  (parent 38)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 111)
  (parent 38)
  (unrealized (0 1) (2 0) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 112)
  (parent 38)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 113)
  (parent 38)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d)))))
  (label 114)
  (parent 38)
  (unrealized (0 1) (2 0) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 115)
  (parent 39)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a)))))
  (label 116)
  (parent 39)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 117)
  (parent 40)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 118)
  (parent 40)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 119)
  (parent 41)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 120)
  (parent 41)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 121)
  (parent 42)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 122)
  (parent 42)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 123)
  (parent 42)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 124)
  (parent 42)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 125)
  (parent 42)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 126)
  (parent 43)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 127)
  (parent 43)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 128)
  (parent 44)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 129)
  (parent 44)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 130)
  (parent 45)
  (unrealized (0 1) (2 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 131)
  (parent 45)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 132)
  (parent 45)
  (unrealized (0 1) (2 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 133)
  (parent 45)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 134)
  (parent 45)
  (unrealized (0 1) (2 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 135)
  (parent 45)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 136)
  (parent 46)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 137)
  (parent 46)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 138)
  (parent 46)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 139)
  (parent 47)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 140)
  (parent 47)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 141)
  (parent 49)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 142)
  (parent 49)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 143)
  (parent 49)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-0 (2 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 144)
  (parent 50)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-2 (2 0)
    (enc a b c nb-2 nb-2 nc-1 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 145)
  (parent 50)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 146)
  (parent 50)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 147)
  (parent 50)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 148)
  (parent 51)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 149)
  (parent 51)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 150)
  (parent 53)
  (unrealized (0 1) (2 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 151)
  (parent 53)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 152)
  (parent 53)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 153)
  (parent 53)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (3 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 154)
  (parent 54)
  (unrealized (0 1) (3 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-0 (3 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 155)
  (parent 54)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (3 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 156)
  (parent 54)
  (unrealized (0 1) (3 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 157)
  (parent 55)
  (unrealized (0 1) (3 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 158)
  (parent 55)
  (unrealized (0 1) (3 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 159)
  (parent 55)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 160)
  (parent 55)
  (unrealized (0 1) (3 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 161)
  (parent 55)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 162)
  (parent 55)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a)))))
  (label 163)
  (parent 56)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 164)
  (parent 57)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 165)
  (parent 58)
  (unrealized (0 1) (3 0) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 166)
  (parent 58)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 167)
  (parent 58)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 168)
  (parent 58)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 169)
  (parent 59)
  (unrealized (0 1) (3 0) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 170)
  (parent 59)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a c d nb (pubk b)) (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a)))))
  (label 171)
  (parent 60)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 172)
  (parent 60)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (4 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 173)
  (parent 62)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 1 resp1 2) nc-0 (4 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 174)
  (parent 62)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) nc-1 (4 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 175)
  (parent 62)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nc-0 (4 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 176)
  (parent 62)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 5 3 resp3 2) nc-1 (4 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d)))))
  (label 177)
  (parent 62)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 178)
  (parent 63)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-2 (4 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 179)
  (parent 63)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 180)
  (parent 63)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-2 (4 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 181)
  (parent 63)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 182)
  (parent 64)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 183)
  (parent 64)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 184)
  (parent 65)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 185)
  (parent 65)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 186)
  (parent 66)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 187)
  (parent 66)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-0 nc-0)) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 188)
  (parent 67)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 3 resp1 2) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 189)
  (parent 67)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nc-1 (4 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 190)
  (parent 67)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 191)
  (parent 67)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 192)
  (parent 67)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 193)
  (parent 68)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 194)
  (parent 69)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 195)
  (parent 69)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 196)
  (parent 69)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-0 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b d nc nc (pubk c))
    (enc a b d nc nb (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a)))))
  (label 197)
  (parent 70)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 198)
  (parent 70)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 199)
  (parent 70)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 200)
  (parent 71)
  (unrealized (0 1) (3 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 201)
  (parent 71)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 202)
  (parent 71)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 203)
  (parent 71)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nd nb text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-0 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b d nc nc (pubk c))
    (enc a b d nc nb (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c)))))
  (label 204)
  (parent 72)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 205)
  (parent 72)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 206)
  (parent 72)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (4 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 207)
  (parent 74)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nc-0 (4 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 208)
  (parent 74)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 5 3 resp3 2) nc-1 (4 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d)))))
  (label 209)
  (parent 74)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 210)
  (parent 75)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 211)
  (parent 75)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 212)
  (parent 76)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-2 (4 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 213)
  (parent 76)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 214)
  (parent 76)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 215)
  (parent 76)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (1 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 216)
  (parent 77)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-3 (pubk c)))))
  (label 217)
  (parent 78)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 218)
  (parent 78)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 219)
  (parent 78)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 220)
  (parent 78)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 221)
  (parent 78)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 222)
  (parent 78)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 223)
  (parent 78)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a b d nc-0 nb-1 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 224)
  (parent 79)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 225)
  (parent 80)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 226)
  (parent 80)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 227)
  (parent 82)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 228)
  (parent 82)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 229)
  (parent 82)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 230)
  (parent 82)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 231)
  (parent 82)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 232)
  (parent 83)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 233)
  (parent 83)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 234)
  (parent 83)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 235)
  (parent 83)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 236)
  (parent 83)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 237)
  (parent 83)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 238)
  (parent 84)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 239)
  (parent 84)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 240)
  (parent 84)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 241)
  (parent 84)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 242)
  (parent 84)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))))
  (label 243)
  (parent 84)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 244)
  (parent 85)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 245)
  (parent 85)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 246)
  (parent 85)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-2 (5 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 247)
  (parent 85)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 248)
  (parent 85)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-2 (5 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 249)
  (parent 85)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 250)
  (parent 86)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 251)
  (parent 86)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 252)
  (parent 86)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))))
  (label 253)
  (parent 86)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 254)
  (parent 86)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 255)
  (parent 86)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 256)
  (parent 87)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 257)
  (parent 87)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 258)
  (parent 87)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 259)
  (parent 87)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 260)
  (parent 87)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 261)
  (parent 87)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 262)
  (parent 88)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 263)
  (parent 88)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (4 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 264)
  (parent 89)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-3 (pubk c)))))
  (label 265)
  (parent 90)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 266)
  (parent 90)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 267)
  (parent 90)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 268)
  (parent 90)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 269)
  (parent 90)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 270)
  (parent 90)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 271)
  (parent 90)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a)))))
  (label 272)
  (parent 91)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 273)
  (parent 91)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a b d nc-0 nb-1 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 274)
  (parent 92)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 275)
  (parent 93)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 276)
  (parent 93)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 277)
  (parent 94)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 278)
  (parent 94)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 279)
  (parent 94)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 280)
  (parent 94)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 281)
  (parent 94)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 282)
  (parent 95)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 283)
  (parent 95)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 284)
  (parent 95)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 285)
  (parent 95)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 286)
  (parent 95)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 287)
  (parent 98)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 288)
  (parent 98)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 289)
  (parent 98)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 290)
  (parent 98)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 291)
  (parent 98)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 4 resp3 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d)))))
  (label 292)
  (parent 98)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 293)
  (parent 99)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 294)
  (parent 99)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 4 resp3 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d)))))
  (label 295)
  (parent 99)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 296)
  (parent 100)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 297)
  (parent 100)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 298)
  (parent 100)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 299)
  (parent 102)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 300)
  (parent 102)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 301)
  (parent 102)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-3 (pubk c)))))
  (label 302)
  (parent 103)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 303)
  (parent 103)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 304)
  (parent 103)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 305)
  (parent 103)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 306)
  (parent 104)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 307)
  (parent 104)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 308)
  (parent 104)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 309)
  (parent 106)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 310)
  (parent 106)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 311)
  (parent 106)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 312)
  (parent 106)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 313)
  (parent 106)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 314)
  (parent 106)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 315)
  (parent 106)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 316)
  (parent 107)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 317)
  (parent 107)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 318)
  (parent 107)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 319)
  (parent 107)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 320)
  (parent 107)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 321)
  (parent 107)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 322)
  (parent 107)
  (unrealized (0 1) (2 0))
  (comment "aborted"))
